@@ -6,8 +6,8 @@
 #define GEMMI_CCP4_HPP_
 
 #include <cassert>
-#include <cmath>     // for NAN, sqrt
-#include <cstdint>   // for uint16_t, uint32_t
+#include <cmath>     // for ceil, fabs, floor, round
+#include <cstdint>   // for uint16_t, int32_t
 #include <cstdio>    // for FILE
 #include <cstring>   // for memcpy
 #include <array>
@@ -25,11 +25,10 @@ namespace gemmi {
 using std::int32_t;
 
 // options for Ccp4<>::setup
-enum class GridSetup {
-  ReorderOnly,  // reorder axes to X, Y, Z
-  ResizeOnly,   // reorder and resize to the whole cell, but no symmetry ops
+enum class MapSetup {
   Full,         // reorder and expand to the whole unit cell
-  FullCheck     // additionally, check consistency of redundant data
+  NoSymmetry,   // reorder and resize to the whole cell, but no symmetry ops
+  ReorderOnly   // reorder axes to X, Y, Z
 };
 
 struct Ccp4Base {
@@ -269,7 +268,7 @@ struct Ccp4 : public Ccp4Base {
     }
   }
 
-  double setup(GridSetup mode, T default_value);
+  double setup(T default_value, MapSetup mode=MapSetup::Full);
   void set_extent(const Box<Fractional>& box);
 
   template<typename Stream>
@@ -301,6 +300,12 @@ struct Ccp4 : public Ccp4Base {
 
 namespace impl {
 
+template<typename From, typename To>
+To translate_map_point(From f) { return static_cast<To>(f); }
+// We convert map 2 to 0 by translating non-zero values to 1.
+template<> inline
+std::int8_t translate_map_point<float,std::int8_t>(float f) { return f != 0; }
+
 template<typename Stream, typename TFile, typename TMem>
 void read_data(Stream& f, std::vector<TMem>& content) {
   if (typeid(TFile) == typeid(TMem)) {
@@ -315,7 +320,7 @@ void read_data(Stream& f, std::vector<TMem>& content) {
       if (!f.read(work.data(), sizeof(TFile) * len))
         fail("Failed to read all the data from the map file.");
       for (size_t j = 0; j < len; ++j)
-        content[i+j] = static_cast<TMem>(work[j]);
+        content[i+j] = translate_map_point<TFile,TMem>(work[j]);
     }
   }
 }
@@ -372,20 +377,8 @@ void Ccp4<T>::read_ccp4_stream(Stream f, const std::string& path) {
   }
 }
 
-namespace impl {
-
-template<typename T> bool is_same(T a, T b) { return a == b; }
-template<> inline bool is_same(float a, float b) {
-  return std::isnan(b) ? std::isnan(a) : a == b;
-}
-template<> inline bool is_same(double a, double b) {
-  return std::isnan(b) ? std::isnan(a) : a == b;
-}
-
-} // namespace impl
-
 template<typename T>
-double Ccp4<T>::setup(GridSetup mode, T default_value) {
+double Ccp4<T>::setup(T default_value, MapSetup mode) {
   double max_error = 0.0;
   if (grid.axis_order == AxisOrder::XYZ || ccp4_header.empty())
     return max_error;
@@ -396,7 +389,7 @@ double Ccp4<T>::setup(GridSetup mode, T default_value) {
   std::array<int, 3> start = header_3i32(5);
   int end[3] = { start[0] + grid.nu, start[1] + grid.nv, start[2] + grid.nw };
   // set new metadata
-  if (mode == GridSetup::ReorderOnly) {
+  if (mode == MapSetup::ReorderOnly) {
     set_header_3i32(5, start[pos[0]], start[pos[1]], start[pos[2]]);
     for (int i = 0; i < 3; ++i) {
       end[i] -= start[i];
@@ -426,12 +419,7 @@ double Ccp4<T>::setup(GridSetup mode, T default_value) {
         full[new_index] = val;
       }
   grid.data = std::move(full);
-  if (mode == GridSetup::Full) {
-    grid.axis_order = AxisOrder::XYZ;
-    grid.symmetrize([&default_value](T a, T b) {
-        return impl::is_same(a, default_value) ? b : a;
-    });
-  } else if (mode == GridSetup::FullCheck) {
+  if (mode == MapSetup::Full) {
     grid.axis_order = AxisOrder::XYZ;
     grid.symmetrize([&max_error, &default_value](T a, T b) {
         if (impl::is_same(a, default_value))
@@ -452,6 +440,8 @@ double Ccp4<T>::setup(GridSetup mode, T default_value) {
 
 template<typename T>
 void Ccp4<T>::set_extent(const Box<Fractional>& box) {
+  if (ccp4_header.empty())
+    fail("set_extent(): no header in the map. Call update_ccp4_header() first");
   if (!full_cell())
     fail("Ccp4::set_extent() works only after setup()");
   if (grid.axis_order != AxisOrder::XYZ)
@@ -463,13 +453,8 @@ void Ccp4<T>::set_extent(const Box<Fractional>& box) {
   int nv = (int)std::floor(box.maximum.y * grid.nv) - v0 + 1;
   int nw = (int)std::floor(box.maximum.z * grid.nw) - w0 + 1;
   // set the data
-  std::vector<T> data((size_t)nu * nv * nw);
-  int idx = 0;
-  for (int w = 0; w < nw; w++) // sections
-    for (int v = 0; v < nv; v++) // rows
-      for (int u = 0; u < nu; u++) // cols
-        data[idx++] = grid.get_value(u0 + u, v0 + v, w0 + w);
-  grid.data = std::move(data);
+  grid.data.resize((size_t)nu * nv * nw);
+  grid.get_subarray(grid.data.data(), {u0, v0, w0}, {nu, nv, nw});
   // and metadata
   grid.nu = nu;
   grid.nv = nv;
