@@ -73,10 +73,10 @@ template<typename T, typename M> std::vector<T> model_subchains(M* model) {
 } // namespace impl
 
 
-// File format with macromolecular model.
-// Unknown = unknown coordinate format (not ChemComp)
-// UnknownAny = any format (coordinate file for a monomer/ligand/chemcomp)
-enum class CoorFormat { Unknown, UnknownAny, Pdb, Mmcif, Mmjson, ChemComp };
+// File format of a macromolecular model. When passed to read_structure():
+// Unknown = guess format from the extension,
+// Detect = guess format from the content.
+enum class CoorFormat { Unknown, Detect, Pdb, Mmcif, Mmjson, ChemComp };
 
 // corresponds to _atom_site.calc_flag in mmCIF
 enum class CalcFlag : signed char { NotSet=0, Determined, Calculated, Dummy };
@@ -163,6 +163,7 @@ struct Residue : public ResidueId {
   static const char* what() { return "Residue"; }
 
   std::string subchain;   // mmCIF _atom_site.label_asym_id
+  std::string entity_id;  // mmCIF _atom_site.label_entity_id
   OptionalNum label_seq;  // mmCIF _atom_site.label_seq_id
   EntityType entity_type = EntityType::Unknown;
   char het_flag = '\0';   // 'A' = ATOM, 'H' = HETATM, 0 = unspecified
@@ -310,7 +311,8 @@ struct ConstResidueSpan : Span<const Residue> {
     if (this->empty())
       throw std::out_of_range("subchain_id(): empty span");
     if (this->size() > 1 && this->front().subchain != this->back().subchain)
-      fail("subchain id varies");
+      fail("subchain id varies in a residue span: ", this->front().subchain,
+           " vs ", this->back().subchain);
     return this->begin()->subchain;
   }
 
@@ -447,13 +449,13 @@ inline ResidueSpan::GroupingProxy ResidueSpan::residue_groups() {
 namespace impl {
 template<typename T, typename Ch> std::vector<T> chain_subchains(Ch* ch) {
   std::vector<T> v;
-  auto span_start = ch->residues.begin();
-  for (auto i = span_start; i != ch->residues.end(); ++i)
-    if (i->subchain != span_start->subchain) {
-      v.push_back(ch->whole().sub(span_start, i));
-      span_start = i;
-    }
-  v.push_back(ch->whole().sub(span_start, ch->residues.end()));
+  for (auto start = ch->residues.begin(); start != ch->residues.end(); ) {
+    auto end = start + 1;
+    while (end != ch->residues.end() && end->subchain == start->subchain)
+      ++end;
+    v.push_back(ch->whole().sub(start, end));
+    start = end;
+  }
   return v;
 }
 } // namespace impl
@@ -482,9 +484,14 @@ struct Chain {
   }
 
   ResidueSpan get_polymer() {
-    return get_residue_span([](const Residue& r) {
-        return r.entity_type == EntityType::Polymer;
-    });
+    auto begin = residues.begin();
+    while (begin != residues.end() && begin->entity_type != EntityType::Polymer)
+      ++begin;
+    auto end = begin;
+    while (end != residues.end() && end->entity_type == EntityType::Polymer
+                                 && end->subchain == begin->subchain)
+      ++end;
+    return ResidueSpan(residues, &*begin, end - begin);
   }
   ConstResidueSpan get_polymer() const {
     return const_cast<Chain*>(this)->get_polymer();
@@ -836,17 +843,17 @@ struct Model {
   const std::vector<Chain>& children() const { return chains; }
 };
 
-inline Entity* find_entity(const std::string& subchain_id,
-                           std::vector<Entity>& entities) {
+inline Entity* find_entity_of_subchain(const std::string& subchain_id,
+                                       std::vector<Entity>& entities) {
   if (!subchain_id.empty())
     for (Entity& ent : entities)
       if (in_vector(subchain_id, ent.subchains))
         return &ent;
   return nullptr;
 }
-inline const Entity* find_entity(const std::string& subchain_id,
-                                 const std::vector<Entity>& entities) {
-  return find_entity(subchain_id, const_cast<std::vector<Entity>&>(entities));
+inline const Entity* find_entity_of_subchain(const std::string& subchain_id,
+                                             const std::vector<Entity>& entities) {
+  return find_entity_of_subchain(subchain_id, const_cast<std::vector<Entity>&>(entities));
 }
 
 struct Structure {
@@ -918,7 +925,7 @@ struct Structure {
   }
 
   Entity* get_entity_of(const ConstResidueSpan& sub) {
-    return sub ? find_entity(sub.subchain_id(), entities) : nullptr;
+    return sub ? find_entity_of_subchain(sub.subchain_id(), entities) : nullptr;
   }
   const Entity* get_entity_of(const ConstResidueSpan& sub) const {
     return const_cast<Structure*>(this)->get_entity_of(sub);
