@@ -21,6 +21,8 @@
 
 namespace gemmi {
 
+typedef cif::Document (*read_cif_func)(const std::string&);
+
 inline void add_distinct_altlocs(const Residue& res, std::string& altlocs) {
   for (const Atom& atom : res.atoms)
     if (atom.altloc && altlocs.find(atom.altloc) == std::string::npos)
@@ -61,6 +63,7 @@ struct ChemLink {
         ++cstr;
       switch (ialpha4_id(cstr)) {
         case ialpha4_id("pept"): return Group::Peptide;
+        case ialpha4_id("l-pe"): return Group::Peptide;
         case ialpha4_id("p-pe"): return Group::PPeptide;
         case ialpha4_id("m-pe"): return Group::MPeptide;
         case ialpha4_id("pyra"): return Group::Pyranose;
@@ -219,6 +222,7 @@ inline ResidueInfo::Kind chemcomp_group_to_kind(const std::string& group) {
     switch (ialpha4_id(str)) {
       case ialpha4_id("non-"): return ResidueInfo::ELS;
       case ialpha4_id("pept"): return ResidueInfo::AA;
+      case ialpha4_id("l-pe"): return ResidueInfo::AA;
       case ialpha4_id("p-pe"): return ResidueInfo::PAA;
       case ialpha4_id("m-pe"): return ResidueInfo::MAA;
       case ialpha4_id("dna"): return ResidueInfo::DNA;
@@ -282,11 +286,13 @@ inline Restraints read_restraint_modifications(const cif::Block& block_) {
   for (auto row : block.find("_chem_mod_bond.",
                              {"function", "atom_id_1", "atom_id_2",
                               "new_type",
-                              "new_value_dist", "new_value_dist_esd"}))
+                              "new_value_dist", "new_value_dist_esd",
+                              "?new_value_dist_nucleus", "?new_value_dist_nucleus_esd"}))
     rt.bonds.push_back({{chem_mod_type(row[0]), row.str(1)}, {1, row.str(2)},
                         bond_type_from_string(row[3]), false,
                         cif::as_number(row[4]), cif::as_number(row[5]),
-                        NAN, NAN});
+                        row.has(6) ? cif::as_number(row[6]) : NAN,
+                        row.has(7) ? cif::as_number(row[7]) : NAN});
   for (auto row : block.find("_chem_mod_angle.",
                              {"function", "atom_id_1",
                               "atom_id_2", "atom_id_3",
@@ -348,19 +354,6 @@ inline void insert_chemmods(const cif::Document& doc,
   }
 }
 
-namespace impl {
-template <typename T>
-T& add_or_set(std::vector<T>& items, typename std::vector<T>::iterator it,
-              const T& x) {
-  if (it == items.end()) {
-    items.push_back(x);
-    return items.back();
-  }
-  *it = x;
-  return *it;
-}
-} // namespace impl
-
 inline void ChemMod::apply_to(ChemComp& chemcomp) const {
   // _chem_mod_atom
   for (const AtomMod& mod : atom_mods) {
@@ -419,7 +412,11 @@ inline void ChemMod::apply_to(ChemComp& chemcomp) const {
     auto it = chemcomp.rt.find_bond(mod.id1.atom, mod.id2.atom);
     switch (mod.id1.comp) {
       case 'a':
-        impl::add_or_set(chemcomp.rt.bonds, it, mod).id1.comp = 1;
+        if (it == chemcomp.rt.bonds.end()) {
+          chemcomp.rt.bonds.push_back(mod);
+          // id1.comp was temporarily set to 'a', set it back to 1
+          chemcomp.rt.bonds.back().id1.comp = 1;
+        }
         break;
       case 'd':
         if (it != chemcomp.rt.bonds.end())
@@ -433,6 +430,10 @@ inline void ChemMod::apply_to(ChemComp& chemcomp) const {
             it->value = mod.value;
           if (!std::isnan(mod.esd))
             it->esd = mod.esd;
+          if (!std::isnan(mod.value_nucleus))
+            it->value_nucleus = mod.value_nucleus;
+          if (!std::isnan(mod.esd_nucleus))
+            it->esd_nucleus = mod.esd_nucleus;
         }
         break;
     }
@@ -443,7 +444,10 @@ inline void ChemMod::apply_to(ChemComp& chemcomp) const {
     auto it = chemcomp.rt.find_angle(mod.id1.atom, mod.id2.atom, mod.id3.atom);
     switch (mod.id1.comp) {
       case 'a':
-        impl::add_or_set(chemcomp.rt.angles, it, mod).id1.comp = 1;
+        if (it == chemcomp.rt.angles.end()) {
+          chemcomp.rt.angles.push_back(mod);
+          chemcomp.rt.angles.back().id1.comp = 1;
+        }
         break;
       case 'd':
         if (it != chemcomp.rt.angles.end())
@@ -466,7 +470,10 @@ inline void ChemMod::apply_to(ChemComp& chemcomp) const {
                                        mod.id3.atom, mod.id4.atom);
     switch (mod.id1.comp) {
       case 'a':
-        impl::add_or_set(chemcomp.rt.torsions, it, mod).id1.comp = 1;
+        if (it == chemcomp.rt.torsions.end()) {
+          chemcomp.rt.torsions.push_back(mod);
+          chemcomp.rt.torsions.back().id1.comp = 1;
+        }
         break;
       case 'd':
         if (it != chemcomp.rt.torsions.end())
@@ -493,7 +500,10 @@ inline void ChemMod::apply_to(ChemComp& chemcomp) const {
                                     mod.id2.atom, mod.id3.atom);
     switch (mod.id1.comp) {
       case 'a':
-        impl::add_or_set(chemcomp.rt.chirs, it, mod).id1.comp = 1;
+        if (it == chemcomp.rt.chirs.end()) {
+          chemcomp.rt.chirs.push_back(mod);
+          chemcomp.rt.chirs.back().id1.comp = 1;
+        }
         break;
       case 'd':
         if (it != chemcomp.rt.chirs.end())
@@ -527,7 +537,6 @@ inline void ChemMod::apply_to(ChemComp& chemcomp) const {
     }
 }
 
-
 struct MonLib {
   cif::Document mon_lib_list;
   std::map<std::string, ChemComp> monomers;
@@ -566,7 +575,7 @@ struct MonLib {
 
   void ensure_unique_link_name(std::string& name) const {
     size_t orig_len = name.size();
-    for (int n = 1; find_link(name) != nullptr; ++n)
+    for (int n = 0; find_link(name) != nullptr; ++n)
       name.replace(orig_len, name.size(), std::to_string(n));
   }
 
@@ -585,8 +594,8 @@ struct MonLib {
 
   bool link_side_matches_residue(const ChemLink::Side& side,
                                  const std::string& res_name) const {
-    if (side.comp == res_name)
-      return true;
+    if (side.comp != "")
+      return side.comp == res_name;
     const ResidueInfo* resinfo = find_residue_info(res_name);
     return resinfo && side.matches_group(ChemLink::group_from_residue_info(*resinfo));
   }
@@ -622,38 +631,36 @@ struct MonLib {
     path += ".cif";
     return path;
   }
+
+  void read_monomer_cif(const std::string& path, read_cif_func read_cif) {
+    mon_lib_list = (*read_cif)(path);
+    for (const cif::Block& block : mon_lib_list.blocks)
+      add_monomer_if_present(block);
+    insert_chemlinks(mon_lib_list, links);
+    insert_chemmods(mon_lib_list, modifications);
+    insert_comp_list(mon_lib_list, residue_infos);
+  }
 };
 
-typedef cif::Document (*read_cif_func)(const std::string&);
-
-inline MonLib read_monomer_cif(const std::string& path,
-                               read_cif_func read_cif) {
-  MonLib monlib;
-  monlib.mon_lib_list = (*read_cif)(path);
-  for (const cif::Block& block : monlib.mon_lib_list.blocks)
-    monlib.add_monomer_if_present(block);
-  insert_chemlinks(monlib.mon_lib_list, monlib.links);
-  insert_chemmods(monlib.mon_lib_list, monlib.modifications);
-  insert_comp_list(monlib.mon_lib_list, monlib.residue_infos);
-  return monlib;
-}
-
 inline MonLib read_monomer_lib(std::string monomer_dir,
                                const std::vector<std::string>& resnames,
                                read_cif_func read_cif,
+                               const std::string& libin="",
                                bool ignore_missing=false) {
   if (monomer_dir.empty())
     fail("read_monomer_lib: monomer_dir not specified.");
   if (monomer_dir.back() != '/' && monomer_dir.back() != '\\')
     monomer_dir += '/';
-  MonLib monlib = read_monomer_cif(monomer_dir + "list/mon_lib_list.cif",
-                                   read_cif);
+  MonLib monlib;
+  if (!libin.empty())
+    monlib.read_monomer_cif(libin, read_cif);
+  monlib.read_monomer_cif(monomer_dir + "list/mon_lib_list.cif", read_cif);
   std::string error;
   for (const std::string& name : resnames) {
     try {
       cif::Document doc = (*read_cif)(monomer_dir + MonLib::relative_monomer_path(name));
       auto cc = make_chemcomp_from_cif(name, doc);
-      monlib.monomers.emplace(name, cc);
+      monlib.monomers.emplace(name, std::move(cc));
     } catch(std::runtime_error& err) {
       if (!ignore_missing)
         error += "The monomer " + name + " could not be read: " + err.what() + ".\n";
