@@ -16,6 +16,7 @@
 #include <set>
 #include <algorithm>     // for all_of
 #include "asudata.hpp"   // for calculate_hkl_value_correlation
+#include "eig3.hpp"      // for eigen_decomposition
 #include "mtz.hpp"       // for Mtz
 #include "xds_ascii.hpp" // for XdsAscii
 #include "atox.hpp"      // for read_word
@@ -25,20 +26,6 @@
 
 namespace gemmi {
 
-inline bool parse_voigt_notation(const char* start, const char* end, SMat33<double>& b) {
-  bool first = true;
-  for (double* u : {&b.u11, &b.u22, &b.u33, &b.u23, &b.u13, &b.u12}) {
-    if (*start != (first ? '(' : ','))
-      return false;
-    first = false;
-    auto result = fast_from_chars(++start, end, *u);
-    if (result.ec != std::errc())
-      return false;
-    start = skip_blank(result.ptr);
-  }
-  return *start == ')';
-}
-
 class MtzToCif {
 public:
   enum Var { Dot=-1, Qmark=-2, Counter=-3, DatasetId=-4, Image=-5 };
@@ -51,12 +38,15 @@ public:
   bool with_history = true;          // write MTZ history in comments
   bool skip_empty = false;           // skip reflections with no values
   bool enable_UB = false;            // write _diffrn_orient_matrix.UB
+  bool write_staraniso_tensor = true; // write _reflns.pdbx_aniso_B_tensor_*
   bool write_special_marker_for_pdb = false;
   int less_anomalous = 0;            // skip (+)/(-) columns even if in spec
   std::string skip_empty_cols;       // columns used to determine "emptiness"
   double wavelength = NAN;           // user-specified wavelength
   int trim = 0;                      // output only reflections -N<=h,k,l<=N
   int free_flag_value = -1;          // -1 = auto: 0 or (if we have >50% of 0's) 1
+  std::string staraniso_version;     // for _software.version in "special_marker"
+  std::string gemmi_run_from;        // added to gemmi as _software.description
 
   static const char** default_spec(bool for_merged) {
     static const char* merged[] = {
@@ -76,11 +66,11 @@ public:
       "& SIG{prev}                L pdbx_F_plus_sigma",
       "? F(-)|FOBS(-)|F-obs(-)    G pdbx_F_minus",
       "& SIG{prev}                L pdbx_F_minus_sigma",
-      "? FREE|RFREE|FREER|FreeR_flag|R-free-flags I status S",
+      "? FREE|RFREE|FREER|FreeR_flag|R-free-flags|FreeRflag I status S",
       "? FWT|2FOFCWT              F pdbx_FWT",
-      "& PHWT|PH2FOFCWT           P pdbx_PHWT",
+      "& PHWT|PH2FOFCWT           P pdbx_PHWT .3f",
       "? DELFWT|FOFCWT            F pdbx_DELFWT",
-      "& DELPHWT|PHDELWT|PHFOFCWT P pdbx_DELPHWT",
+      "& DELPHWT|PHDELWT|PHFOFCWT P pdbx_DELPHWT .3f",
       nullptr
     };
     static const char* unmerged[] = {
@@ -99,40 +89,11 @@ public:
     return for_merged ? merged : unmerged;
   }
 
-  void write_cif(const Mtz& mtz, const Mtz* mtz2, std::ostream& os);
+  void write_cif(const Mtz& mtz, const Mtz* mtz2,
+                 SMat33<double>* staraniso_b, std::ostream& os);
   void write_cif_from_xds(const XdsAscii& xds, std::ostream& os);
 
-  void check_staraniso(const Mtz& mtz, std::ostream& out) {
-    size_t hlen = mtz.history.size();
-    for (size_t i = 0; i != hlen; ++i)
-      if (mtz.history[i].find("STARANISO") != std::string::npos) {
-        size_t version_pos = mtz.history[i].find("version:");
-        if (version_pos != std::string::npos)
-          staraniso_version = read_word(mtz.history[i].c_str() + version_pos + 8);
-        out << "History in merged MTZ includes scaling with STARANISO "
-            << staraniso_version << ".\n";
-        // StarAniso 2.3.74 (24-Apr-2021) and later write B tensor in history
-        bool got_b = false;
-        for (size_t j = i+1; j < std::min(i+4, hlen); ++j) {
-          const std::string& line = mtz.history[j];
-          if (starts_with(line, "B=(")) {
-            got_b = parse_voigt_notation(line.c_str() + 2,
-                                         line.c_str() + line.size(),
-                                         staraniso_b);
-            if (!got_b)
-              fail("failed to parse tensor Voigt notation: " + line);
-            break;
-          }
-        }
-        if (!got_b) {
-          out << "StarAniso B tensor not found. Intensities won't be checked.\n";
-          staraniso_b.u11 = NAN;
-        }
-        break;
-      }
-  }
-
-  const SMat33<double>& get_staraniso_b() const { return staraniso_b; }
+  void write_staraniso_b_in_mmcif(const SMat33<double>& b, char* buf, std::ostream& os);
 
 private:
   // describes which MTZ column is to be translated to what mmCIF column
@@ -146,32 +107,29 @@ private:
 
   // data corresponding to one sweep (dataset) in unmerged MTZ file
   struct SweepData {
-    int id;
-    int batch_count;
-    int offset;
-    const Mtz::Batch* first_batch;
-    const Mtz::Dataset* dataset;
+    int id = 0;
+    int batch_count = 0;
+    int offset = 0;
+    int crystal_id = 1;
+    const Mtz::Batch* first_batch = nullptr;
+    const Mtz::Dataset* dataset = nullptr;
   };
 
   std::vector<SweepData> sweeps;
   std::unordered_map<int, int> sweep_indices;
 
-  std::vector<Trans> recipe;
-
-  SMat33<double> staraniso_b = {0., 0., 0., 0., 0., 0.};
-  std::string staraniso_version;
-
-  const Trans* get_status_translation() const {
+  static const Trans* get_status_translation(const std::vector<Trans>& recipe) {
     for (const Trans& t: recipe)
       if (t.is_status)
         return &t;
     return nullptr;
   }
 
-  void gather_sweep_data(const Mtz& mtz) {
+  bool gather_sweep_data(const Mtz& mtz) {
+    bool ok = true;
     int prev_number = INT_MIN;
     int prev_dataset = INT_MIN;
-    SweepData sweep{0, 0, 0, nullptr, nullptr};
+    SweepData sweep;
     for (const Mtz::Batch& batch : mtz.batches) {
       int dataset_id = batch.dataset_id();
       if (dataset_id != prev_dataset || batch.number != prev_number + 1) {
@@ -186,6 +144,7 @@ private:
           sweep.dataset = &mtz.dataset(dataset_id);
         } catch (std::runtime_error&) {
           sweep.dataset = nullptr;
+          ok = false;
           mtz.warn("Reference to absent dataset: " + std::to_string(dataset_id));
         }
       }
@@ -195,6 +154,7 @@ private:
     }
     if (sweep.id != 0)
       sweeps.push_back(sweep);
+    return ok;
   }
 
   // Get the first (non-zero) DWAVEL corresponding to an intensity, amplitude
@@ -266,16 +226,16 @@ private:
     bool discard_next_line = false;
   };
 
-  void prepare_recipe(const Mtz& mtz) {
+  void prepare_recipe(const Mtz& mtz, std::vector<Trans>& recipe) const {
     recipe.clear();
     SpecParserState state;
     if (!spec_lines.empty()) {
       for (const std::string& line : spec_lines)
-        parse_spec_line(line.c_str(), mtz, state);
+        parse_spec_line(recipe, line.c_str(), mtz, state);
     } else {
       const char** lines = default_spec(/*for_merged=*/mtz.batches.empty());
       for (; *lines != nullptr; ++lines)
-        parse_spec_line(*lines, mtz, state);
+        parse_spec_line(recipe, *lines, mtz, state);
     }
     if (recipe.empty())
       fail("empty translation recipe");
@@ -295,7 +255,8 @@ private:
   }
 
   // adds results to recipe
-  void parse_spec_line(const char* line, const Mtz& mtz, SpecParserState& state) {
+  void parse_spec_line(std::vector<Trans>& recipe, const char* line,
+                       const Mtz& mtz, SpecParserState& state) const {
     Trans tr;
     const char* p = line;
     if (*p == '&') {
@@ -386,20 +347,28 @@ private:
     if (!write_special_marker_for_pdb)
       return;
     os << "### IF YOU MODIFY THIS FILE, REMOVE THIS SIGNATURE: ###\n";
+    std::string desc;
+    if (!gemmi_run_from.empty())
+      desc = " 'run from " + gemmi_run_from + "'";
     if (!merged || staraniso_version.empty()) {
       os << "_software.pdbx_ordinal 1\n"
             "_software.classification 'data extraction'\n"
             "_software.name gemmi\n"
             "_software.version " GEMMI_VERSION "\n";
+      if (!desc.empty())
+        os << "_software.description" << desc << '\n';
     } else {
       os << "loop_\n"
             "_software.pdbx_ordinal\n"
             "_software.classification\n"
             "_software.name\n"
-            "_software.version\n"
-            "1 'data extraction' gemmi " GEMMI_VERSION "\n";
+            "_software.version\n";
+      if (!desc.empty())
+        os << "_software.description\n";
+      os << "1 'data extraction' gemmi " GEMMI_VERSION << desc << '\n';
       // STARANISO here tells that intensities were scaled anisotropically.
-      os << "2 'data scaling' STARANISO '" << staraniso_version << "'\n";
+      os << "2 'data scaling' STARANISO '" << staraniso_version
+         << (!desc.empty() ? "' .\n" : "'\n");
     }
     os << "_pdbx_audit_conform.dict_name mmcif_pdbx.dic\n"
           "_pdbx_audit_conform.dict_version 5.339\n"
@@ -412,9 +381,37 @@ private:
                                const SpaceGroup* sg,
                                char* buf, std::ostream& os) const;
 
-  void write_main_loop(const Mtz& mtz, char* buf, std::ostream& os);
+  void write_main_loop(const Mtz& mtz, const std::vector<Trans>& recipe,
+                       char* buf, std::ostream& os);
 };
 
+// remove '_dataset_name' that can appended to column names in ccp4i
+inline void remove_appendix_from_column_names(Mtz& mtz, std::ostream& out) {
+  std::string appendix;
+  for (char type : {'J', 'F'}) {  // intensity, amplitude
+    auto cols = mtz.columns_with_type(type);
+    // Remove the appendix only in clear cases. If mtz has multiple
+    // intensity columns they could have IMEAN_this and IMEAN.
+    if (cols.size() == 1) {
+      auto pos = cols[0]->label.find('_');
+      if (pos == std::string::npos)
+        return;
+      appendix = cols[0]->label.substr(pos);
+      break;
+    }
+  }
+  out << "Ignoring '" << appendix << "' appended to column names.\n";
+  for (Mtz::Column& col : mtz.columns) {
+    size_t from_end  = appendix.size();
+    // the appendix can before (+)/(-), i.e. I_appendix(+)
+    if (!col.label.empty() && col.label.back() == ')')
+      from_end += 3;
+    if (from_end < col.label.size() &&
+        col.label.compare(col.label.size() - from_end, appendix.size(), appendix) == 0)
+      col.label.erase(col.label.size() - from_end, appendix.size());
+  }
+}
+
 inline bool validate_merged_mtz_deposition_columns(const Mtz& mtz, std::ostream& out) {
   bool ok = true;
   if (!mtz.rfree_column()) {
@@ -441,8 +438,7 @@ inline bool validate_merged_mtz_deposition_columns(const Mtz& mtz, std::ostream&
 
 // note: both mi and ui get modified
 inline bool validate_merged_intensities(Intensities& mi, Intensities& ui,
-                                        const SMat33<double>& scale_aniso_b,
-                                        std::ostream& out) {
+                                        bool relaxed_check, std::ostream& out) {
   // XDS files have 4 significant digits. Using accuracy 5x the precision.
   const double max_diff = 0.005;
   out << "Checking if both files match...\n";
@@ -461,6 +457,7 @@ inline bool validate_merged_intensities(Intensities& mi, Intensities& ui,
       out << "(in the future, this app may recognize compatible space groups\n"
              "and reindex unmerged data if needed; for now, it's on you)\n";
   }
+
   auto eq = [](double x, double y, double rmsd) { return std::fabs(x - y) < rmsd + 0.02; };
   if(eq(mi.unit_cell.a,     ui.unit_cell.a,     ui.unit_cell_rmsd[0]) &&
      eq(mi.unit_cell.b,     ui.unit_cell.b,     ui.unit_cell_rmsd[1]) &&
@@ -494,15 +491,10 @@ inline bool validate_merged_intensities(Intensities& mi, Intensities& ui,
   out << "Merged reflections: " << mi_size1 << ' ' << mi.type_str()
       << " (" << mi.data.size() << " w/o sysabs)\n";
 
-  bool relaxed_check = std::isnan(scale_aniso_b.u11);
-
-  if (!relaxed_check && !scale_aniso_b.all_zero()) {
+  if (mi.staraniso_b.ok()) {
     out << "Taking into account the anisotropy tensor that was used for scaling.\n";
-    for (Intensities::Refl& refl : ui.data) {
-      Vec3 hkl(refl.hkl[0], refl.hkl[1], refl.hkl[2]);
-      Vec3 s = ui.unit_cell.frac.mat.multiply(hkl);
-      refl.value *= std::exp(0.5 * scale_aniso_b.r_u_r(s));
-    }
+    for (Intensities::Refl& refl : ui.data)
+      refl.value *= mi.staraniso_b.scale(refl.hkl, ui.unit_cell);
   }
 
   // first pass - calculate CC and scale
@@ -525,23 +517,25 @@ inline bool validate_merged_intensities(Intensities& mi, Intensities& ui,
   };
   while (r1 != ui.data.end() && r2 != mi.data.end()) {
     if (r1->hkl == r2->hkl && r1->isign == r2->isign) {
-      double value1 = scale * r1->value;
-      double sigma1 = scale * r1->sigma; // is this approximately correct
-      double sq_value_max = std::max(sq(value1), sq(r2->value));
-      double sq_diff = sq(value1 - r2->value);
-      // Intensities may happen to be rounded to two decimal places,
-      // so if the absolute difference is <0.01 it's OK.
-      if (!relaxed_check && sq_diff > 1e-4 && sq_diff > sq(max_diff) * sq_value_max) {
-        if (differ_count == 0) {
-          out << "First difference: " << refln_str(*r1)
-              << ' ' << value1 << " vs " << r2->value << '\n';
-        }
-        ++differ_count;
-        double weighted_sq_diff = sq_diff / (sq(sigma1) + sq(r2->sigma));
-        if (weighted_sq_diff > max_weighted_sq_diff) {
-          max_weighted_sq_diff = weighted_sq_diff;
-          max_diff_r1 = &*r1;
-          max_diff_r2 = &*r2;
+      if (!relaxed_check) {
+        double value1 = scale * r1->value;
+        double sigma1 = scale * r1->sigma; // is this approximately correct
+        double sq_max = std::max(sq(value1), sq(r2->value));
+        double sq_diff = sq(value1 - r2->value);
+        // Intensities may happen to be rounded to two decimal places,
+        // so if the absolute difference is <0.01 it's OK.
+        if (sq_diff > 1e-4 && sq_diff > sq(max_diff) * sq_max) {
+          if (differ_count == 0) {
+            out << "First difference: " << r1->hkl_label()
+                << ' ' << value1 << " vs " << r2->value << '\n';
+          }
+          ++differ_count;
+          double weighted_sq_diff = sq_diff / (sq(sigma1) + sq(r2->sigma));
+          if (weighted_sq_diff > max_weighted_sq_diff) {
+            max_weighted_sq_diff = weighted_sq_diff;
+            max_diff_r1 = &*r1;
+            max_diff_r2 = &*r2;
+          }
         }
       }
       ++r1;
@@ -586,7 +580,68 @@ inline bool validate_merged_intensities(Intensities& mi, Intensities& ui,
   return ok;
 }
 
-inline void MtzToCif::write_cif(const Mtz& mtz, const Mtz* mtz2, std::ostream& os) {
+#define WRITE(...) os.write(buf, gf_snprintf(buf, 255, __VA_ARGS__))
+
+// Reorder eigenvalues and change signs of eigenvectors in the STARANISO way.
+// It minimises the rotation angle from the basis vectors to the eigenvectors,
+// which is equivalent to maximising the trace of the eigenvector matrix.
+inline void reorder_staraniso_eigensystem(Mat33& vectors, double (&values)[3]) {
+  const int8_t permut[6][3] = {{0,1,2}, {1,2,0}, {2,0,1}, {1,0,2}, {2,1,0}, {0,2,1}};
+  const int8_t signs[8][3] = {{1,1,1}, {1,-1,-1}, {-1,1,-1}, {-1,-1,1},
+                              {-1,-1,-1}, {-1,1,1}, {1,-1,1}, {1,1,-1}};
+  double max_trace = -INFINITY;
+  int permut_pos = 0, sign_pos = 0;
+  bool det_neg = std::signbit(vectors.determinant());
+  for (int i = 0; i < 6; ++i) {
+    int jbase = det_neg == (i > 2) ? 0 : 4;
+    const int8_t (&p)[3] = permut[i];
+    for (int j = jbase; j < jbase+4; ++j) {
+      double trace = 0.;
+      for (int k = 0; k < 3; ++k)
+        trace += signs[j][k] * vectors[k][p[k]];
+      if (trace > max_trace) {
+        max_trace = trace;
+        permut_pos = i;
+        sign_pos = j;
+      }
+    }
+  }
+  const int8_t (&p)[3] = permut[permut_pos];
+  double tmp[3];
+  for (int i = 0; i < 3; ++i) {
+    for (int j = 0; j < 3; ++j)
+      tmp[j] = signs[sign_pos][j] * vectors.a[i][p[j]];
+    std::memcpy(vectors.a[i], tmp, sizeof(tmp));
+  }
+  for (int j = 0; j < 3; ++j)
+    tmp[j] = values[p[j]];
+  std::memcpy(values, tmp, sizeof(tmp));
+}
+
+inline void MtzToCif::write_staraniso_b_in_mmcif(const SMat33<double>& b,
+                                                 char* buf, std::ostream& os) {
+  double eigenvalues[3];
+  Mat33 eigenvectors = eigen_decomposition(b, eigenvalues);
+  reorder_staraniso_eigensystem(eigenvectors, eigenvalues);
+  // three mandatory items in _reflns
+  os << "\n_reflns.entry_id " << entry_id
+     << "\n_reflns.pdbx_ordinal 1"
+        "\n_reflns.pdbx_diffrn_id 1";
+  const char* prefix = "\n_reflns.pdbx_aniso_B_tensor_eigen";
+  for (int i = 0; i < 3; ++i) {
+    // According to the mmCIF spec eigenvalues must be positive.
+    // But we don't have the original eigenvalues, we have values after
+    // subtracting the smallest eigenvalue. So at least one of them is 0.
+    // As a workaround, add an arbitrary number to all eigenvalues.
+    WRITE("%svalue_%d %.5g", prefix, i+1, eigenvalues[i] + 1.0);
+    for (int j = 0; j < 3; ++j)
+      WRITE("%svector_%d_ortho[%d] %.5g", prefix, i+1, j+1, eigenvectors[j][i]);
+  }
+  os << '\n';
+}
+
+inline void MtzToCif::write_cif(const Mtz& mtz, const Mtz* mtz2,
+                                SMat33<double>* staraniso_b, std::ostream& os) {
   if (mtz2 && mtz.is_merged() == mtz2->is_merged())
     fail("If two MTZ files are given, one must be merged and one unmerged,\n"
          "got two ", mtz.is_merged() ? "merged" : "unmerged");
@@ -594,7 +649,6 @@ inline void MtzToCif::write_cif(const Mtz& mtz, const Mtz* mtz2, std::ostream& o
   const Mtz* unmerged = mtz.is_merged() ? mtz2 : &mtz;
 
   char buf[256];
-#define WRITE(...) os.write(buf, gf_snprintf(buf, 255, __VA_ARGS__))
   if (with_comments) {
     os << "# Converted by gemmi-mtz2cif " GEMMI_VERSION "\n";
     for (const Mtz* m : {merged, unmerged})
@@ -613,43 +667,65 @@ inline void MtzToCif::write_cif(const Mtz& mtz, const Mtz* mtz2, std::ostream& o
 
   write_special_marker_if_requested(os, merged);
 
-  if (unmerged) {
-    os << "_exptl_crystal.id 1\n\n";
-    bool scaled = (unmerged->column_with_label("SCALEUSED") != nullptr);
-    gather_sweep_data(*unmerged);
+  std::vector<Trans> recipe;
+  if (merged)
+    prepare_recipe(*merged, recipe);
 
-    os << "loop_\n_diffrn.id\n_diffrn.crystal_id\n_diffrn.details\n";
+  if (unmerged) {
+    bool ok = gather_sweep_data(*unmerged);
+    std::set<const Mtz::Dataset*> used_datasets;
+    std::vector<std::string> crystal_names;
+    // Prepare used_datasets, crystal_names and set SweepData::crystal_id.
+    for (SweepData& sweep : sweeps)
+      if (sweep.dataset) {
+        used_datasets.insert(sweep.dataset);
+        if (ok) {
+          auto it = std::find(crystal_names.begin(), crystal_names.end(),
+                              sweep.dataset->crystal_name);
+          sweep.crystal_id = int(it - crystal_names.begin()) + 1;
+          if (it == crystal_names.end())
+            crystal_names.push_back(sweep.dataset->crystal_name);
+        }
+      }
+
+    if (crystal_names.empty()) {
+      os << "_exptl_crystal.id 1\n";
+    } else {
+      os << "loop_\n_exptl_crystal.id\n";
+      for (size_t i = 0; i < crystal_names.size(); ++i)
+        os << i+1 << " # " << crystal_names[i] << '\n';
+    }
+
+    bool scaled = (unmerged->column_with_label("SCALEUSED") != nullptr);
+
+    os << "\nloop_\n"
+          "_diffrn.id\n_diffrn.crystal_id\n_diffrn.details\n";
     for (const SweepData& sweep : sweeps) {
-      os << sweep.id << " 1 '";
+      os << sweep.id << ' ' << sweep.crystal_id << " '";
       if (scaled)
         os << "scaled ";
       os << "unmerged data'\n";
     }
-    os << '\n';
 
-    os << "loop_\n"
+    os << "\nloop_\n"
           "_diffrn_measurement.diffrn_id\n"
           "_diffrn_measurement.details\n";
     for (const SweepData& sweep : sweeps)
       os << sweep.id << " '" << sweep.batch_count << " frames'\n";
-    os << '\n';
 
-    os << "loop_\n"
+    os << "\nloop_\n"
           "_diffrn_radiation.diffrn_id\n"
           "_diffrn_radiation.wavelength_id\n";
-    std::set<const Mtz::Dataset*> used_datasets;
     for (const SweepData& sweep : sweeps) {
       os << sweep.id << ' ';
-      if (sweep.dataset) {
-        used_datasets.insert(sweep.dataset);
-        os << sweep.dataset->id << '\n';
-      } else {
-        os << "?\n";
-      }
+      if (sweep.dataset)
+        os << sweep.dataset->id;
+      else
+        os << '?';
+      os << '\n';
     }
-    os << '\n';
 
-    os << "loop_\n"
+    os << "\nloop_\n"
           "_diffrn_radiation_wavelength.id\n"
           "_diffrn_radiation_wavelength.wavelength\n";
     for (const Mtz::Dataset* ds : used_datasets)
@@ -679,12 +755,10 @@ inline void MtzToCif::write_cif(const Mtz& mtz, const Mtz* mtz2, std::ostream& o
       }
       os << '\n';
     }
-  } else {
+  } else {  // not unmerged, i.e. only merged data
     double w = std::isnan(wavelength) ? get_wavelength(mtz, recipe) : wavelength;
     if (w > 0.)
-      os << "_diffrn_radiation.diffrn_id 1\n"
-         << "_diffrn_radiation.wavelength_id 1\n"
-         << "_diffrn_radiation_wavelength.id 1\n"
+      os << "_diffrn_radiation_wavelength.id 1\n"
          << "_diffrn_radiation_wavelength.wavelength " << to_str(w) << "\n\n";
   }
 
@@ -696,14 +770,19 @@ inline void MtzToCif::write_cif(const Mtz& mtz, const Mtz* mtz2, std::ostream& o
     write_cell_and_symmetry(cell, rmsds, mtz.spacegroup, buf, os);
   }
 
+  if (write_staraniso_tensor && staraniso_b)
+    write_staraniso_b_in_mmcif(*staraniso_b, buf, os);
+
   if (merged)
-    write_main_loop(*merged, buf, os);
-  if (unmerged)
-    write_main_loop(*unmerged, buf, os);
+    write_main_loop(*merged, recipe, buf, os);
+  if (unmerged) {
+    prepare_recipe(*unmerged, recipe);
+    write_main_loop(*unmerged, recipe, buf, os);
+  }
 }
 
-inline void MtzToCif::write_main_loop(const Mtz& mtz, char* buf, std::ostream& os) {
-  prepare_recipe(mtz);
+inline void MtzToCif::write_main_loop(const Mtz& mtz, const std::vector<Trans>& recipe,
+                                      char* buf, std::ostream& os) {
   // prepare indices
   std::vector<int> value_indices;  // used for --skip_empty
   std::vector<int> sigma_indices;  // used for status 'x'
@@ -746,7 +825,7 @@ inline void MtzToCif::write_main_loop(const Mtz& mtz, char* buf, std::ostream& o
   if (free_flag_value < 0) {
     // CCP4 uses flags 0,...N-1 (usually N=20), with default free set 0
     // PHENIX uses 0/1 flags with free set 1
-    if (const Trans* tr_status = get_status_translation()) {
+    if (const Trans* tr_status = get_status_translation(recipe)) {
       int count = 0;
       for (float val : mtz.columns[tr_status->col_idx])
         if (val == 0.f)
@@ -848,9 +927,9 @@ inline void MtzToCif::write_cif_from_xds(const XdsAscii& xds, std::ostream& os)
 
   write_special_marker_if_requested(os, false);
 
-  os << "_exptl_crystal.id 1\n\n";
+  os << "_exptl_crystal.id 1\n";
 
-  os << "loop_\n_diffrn.id\n_diffrn.crystal_id\n_diffrn.details\n";
+  os << "\nloop_\n_diffrn.id\n_diffrn.crystal_id\n_diffrn.details\n";
   for (const XdsAscii::Iset& iset : xds.isets)
     // We could write iset.input_info as details, but then local paths
     // could end up in the deposition.
@@ -943,22 +1022,22 @@ inline void MtzToCif::write_cell_and_symmetry(const UnitCell& cell, double* rmsd
                                               const SpaceGroup* sg,
                                               char* buf, std::ostream& os) const {
   os << "_cell.entry_id " << entry_id << '\n';
-  WRITE("_cell.length_a    %8.3f\n", cell.a);
+  WRITE("_cell.length_a    %8.4f\n", cell.a);
   if (rmsds && rmsds[0] != 0.)
     WRITE("_cell.length_a_esd %7.3f\n", rmsds[0]);
-  WRITE("_cell.length_b    %8.3f\n", cell.b);
+  WRITE("_cell.length_b    %8.4f\n", cell.b);
   if (rmsds && rmsds[1] != 0.)
     WRITE("_cell.length_b_esd %7.3f\n", rmsds[1]);
-  WRITE("_cell.length_c    %8.3f\n", cell.c);
+  WRITE("_cell.length_c    %8.4f\n", cell.c);
   if (rmsds && rmsds[2] != 0.)
     WRITE("_cell.length_c_esd %7.3f\n", rmsds[2]);
-  WRITE("_cell.angle_alpha %8.3f\n", cell.alpha);
+  WRITE("_cell.angle_alpha %8.4f\n", cell.alpha);
   if (rmsds && rmsds[3] != 0.)
     WRITE("_cell.angle_alpha_esd %7.3f\n", rmsds[3]);
-  WRITE("_cell.angle_beta  %8.3f\n", cell.beta);
+  WRITE("_cell.angle_beta  %8.4f\n", cell.beta);
   if (rmsds && rmsds[4] != 0.)
     WRITE("_cell.angle_beta_esd %8.3f\n", rmsds[4]);
-  WRITE("_cell.angle_gamma %8.3f\n", cell.gamma);
+  WRITE("_cell.angle_gamma %8.4f\n", cell.gamma);
   if (rmsds && rmsds[5] != 0.)
     WRITE("_cell.angle_gamma_esd %7.3f\n", rmsds[5]);
   if (sg) {
