@@ -15,6 +15,8 @@
 
 namespace gemmi {
 
+enum class DataType { Unknown, Unmerged, Mean, Anomalous };
+
 struct ReflnBlock {
   cif::Block block;
   std::string entry_id;
@@ -193,7 +195,7 @@ inline ReflnBlock hkl_cif_as_refln_block(cif::Block& block) {
   rb.entry_id = rb.block.name;
   impl::set_cell_from_mmcif(rb.block, rb.cell, /*mmcif=*/false);
   const char* hm_tag = "_symmetry_space_group_name_H-M";
-  if (const std::string* hm = block.find_value(hm_tag))
+  if (const std::string* hm = rb.block.find_value(hm_tag))
     rb.spacegroup = find_spacegroup_by_name(cif::as_string(*hm),
                                             rb.cell.alpha, rb.cell.gamma);
   rb.cell.set_cell_images_from_spacegroup(rb.spacegroup);
