@@ -256,12 +256,13 @@ void add_cif_atoms(const Structure& st, cif::Block& block, bool use_group_pdb) {
     block.find_mmcif_category("_atom_site_anisotrop.").erase();
   } else {
     cif::Loop& aniso_loop = block.init_mmcif_loop("_atom_site_anisotrop.", {
-                                    "id", "U[1][1]", "U[2][2]", "U[3][3]",
-                                    "U[1][2]", "U[1][3]", "U[2][3]"});
+                                  "id", "type_symbol", "U[1][1]", "U[2][2]",
+                                  "U[3][3]", "U[1][2]", "U[1][3]", "U[2][3]"});
     std::vector<std::string>& aniso_val = aniso_loop.values;
     aniso_val.reserve(aniso_loop.tags.size() * aniso.size());
     for (const auto& a : aniso) {
       aniso_val.emplace_back(std::to_string(a.first));
+      aniso_val.emplace_back(a.second->element.uname());
       aniso_val.emplace_back(to_str(a.second->aniso.u11));
       aniso_val.emplace_back(to_str(a.second->aniso.u22));
       aniso_val.emplace_back(to_str(a.second->aniso.u33));
@@ -1060,7 +1061,7 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
   if (groups.conn)
     impl::write_struct_conn(st, block);
 
-  if ( groups.cis) {  // _struct_mon_prot_cis
+  if (groups.cis) {  // _struct_mon_prot_cis
     cif::Loop& prot_cis_loop = block.init_mmcif_loop("_struct_mon_prot_cis.",
         {"pdbx_id", "pdbx_PDB_model_num", "label_asym_id", "label_seq_id",
          "auth_asym_id", "auth_seq_id", "pdbx_PDB_ins_code",
@@ -1071,7 +1072,7 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
           if (res.is_cis)
             prot_cis_loop.add_row({to_string(prot_cis_loop.length()+1),
                                    model.name, impl::subchain_or_dot(res),
-                                   res.label_seq.str(), impl::qchain(chain.name),
+                                   res.label_seq.str('.'), impl::qchain(chain.name),
                                    res.seqid.num.str(), impl::pdbx_icode(res),
                                    res.name, "."});
   }
