@@ -5,6 +5,8 @@
 #ifndef GEMMI_TOPO_HPP_
 #define GEMMI_TOPO_HPP_
 
+#include <map>           // for multimap
+#include <ostream>       // for ostream
 #include "chemcomp.hpp"  // for ChemComp
 #include "monlib.hpp"    // for MonLib
 #include "model.hpp"     // for Residue, Atom
@@ -72,27 +74,29 @@ struct Topo {
     }
   };
 
-  enum class Provenance { None, PrevLink, Monomer, NextLink, ExtraLink };
   enum class RKind { Bond, Angle, Torsion, Chirality, Plane };
   struct Rule {
-    Provenance provenance;
     RKind rkind;
     size_t index; // index in the respective vector (bonds, ...) in Topo
   };
 
+  struct Link {
+    std::string link_id;
+    Residue* res1 = nullptr;
+    Residue* res2 = nullptr;
+    std::vector<Rule> link_rules;
+    // altloc is used only for ChainInfo::extras, not for ResInfo::prev
+    char alt1 = '\0';
+    char alt2 = '\0';
+  };
+
   struct ResInfo {
     Residue* res;
-    struct Prev {
-      std::string link;
-      int idx; // relative to current index
-      const ResInfo* get(const ResInfo* t) const { return t + idx; }
-      ResInfo* get(ResInfo* t) const { return t + idx; }
-    };
     // in case of microheterogeneity we may have 2+ previous residues
-    std::vector<Prev> prev;
+    std::vector<Link> prev;
     std::vector<std::string> mods;
     ChemComp chemcomp;
-    std::vector<Rule> rules;
+    std::vector<Rule> monomer_rules;
 
     ResInfo(Residue* r) : res(r) {}
     void add_mod(const std::string& m) {
@@ -101,16 +105,18 @@ struct Topo {
     }
   };
 
+  // corresponds to a sub-chain
   struct ChainInfo {
-    std::string name;
+    const Chain& chain_ref;
+    std::string subchain_name;
     std::string entity_id;
     bool polymer;
     PolymerType polymer_type;
     std::vector<ResInfo> res_infos;
 
-    void initialize(ResidueSpan& subchain, const Entity* ent);
+    ChainInfo(ResidueSpan& subchain, const Chain& chain, const Entity* ent);
     void setup_polymer_links();
-    void add_refmac_builtin_modifications();
+    void disable_polymer_link(const Connection& conn);
     struct RGroup {
       std::vector<ResInfo>::iterator begin, end;
     };
@@ -122,15 +128,6 @@ struct Topo {
     }
   };
 
-  struct ExtraLink {
-    Residue* res1;
-    Residue* res2;
-    char alt1 = '\0';
-    char alt2 = '\0';
-    std::string link_id;
-    std::vector<Rule> rules;
-  };
-
   template<typename T>
   static int has_atom(const Atom* a, const T& t) {
     for (int i = 0; (size_t) i != t.atoms.size(); ++i)
@@ -139,8 +136,9 @@ struct Topo {
     return -1;
   }
 
+  std::ostream* warnings = nullptr;
   std::vector<ChainInfo> chain_infos;
-  std::vector<ExtraLink> extras;
+  std::vector<Link> extras;
 
   // Restraints applied to Model
   std::vector<Bond> bonds;
@@ -149,6 +147,9 @@ struct Topo {
   std::vector<Chirality> chirs;
   std::vector<Plane> planes;
 
+  std::multimap<const Atom*, Bond*> bond_index;
+  std::multimap<const Atom*, Angle*> angle_index;
+
   ResInfo* find_resinfo(const Residue* res) {
     for (ChainInfo& ci : chain_infos)
       for (ResInfo& ri : ci.res_infos)
@@ -157,21 +158,34 @@ struct Topo {
     return nullptr;
   }
 
+  Bond* first_bond_in_link(Link& link) {
+    for (const Rule& rule : link.link_rules)
+      if (rule.rkind == RKind::Bond)
+        return &bonds[rule.index];
+    return nullptr;
+  }
+
   const Restraints::Bond* take_bond(const Atom* a, const Atom* b) const {
-    for (const Bond& bond : bonds)
-      if ((bond.atoms[0] == a && bond.atoms[1] == b) ||
-          (bond.atoms[0] == b && bond.atoms[1] == a))
-        return bond.restr;
+    auto range = bond_index.equal_range(a);
+    for (auto i = range.first; i != range.second; ++i) {
+      const Bond* bond = i->second;
+      if ((bond->atoms[0] == b && bond->atoms[1] == a) ||
+          (bond->atoms[1] == b && bond->atoms[0] == a))
+        return bond->restr;
+    }
     return nullptr;
   }
 
   const Restraints::Angle* take_angle(const Atom* a,
                                       const Atom* b,
                                       const Atom* c) const {
-    for (const Angle& ang : angles)
-      if (ang.atoms[1] == b && ((ang.atoms[0] == a && ang.atoms[2] == c) ||
-                                (ang.atoms[0] == c && ang.atoms[2] == a)))
-        return ang.restr;
+    auto range = angle_index.equal_range(b);
+    for (auto i = range.first; i != range.second; ++i) {
+      const Angle* ang = i->second;
+      if ((ang->atoms[0] == a && ang->atoms[2] == c) ||
+          (ang->atoms[0] == c && ang->atoms[2] == a))
+        return ang->restr;
+    }
     return nullptr;
   }
 
@@ -209,12 +223,11 @@ struct Topo {
       altlocs += altloc;
 
     std::vector<Rule> rules;
-    Provenance pro = Provenance::None;
     for (const Restraints::Bond& bond : rt.bonds)
       for (char alt : altlocs)
         if (Atom* at1 = bond.id1.get_from(res, res2, alt))
           if (Atom* at2 = bond.id2.get_from(res, res2, alt)) {
-            rules.push_back({pro, RKind::Bond, bonds.size()});
+            rules.push_back({RKind::Bond, bonds.size()});
             bonds.push_back({&bond, {{at1, at2}}});
             if (!at1->altloc && !at2->altloc)
               break;
@@ -224,7 +237,7 @@ struct Topo {
         if (Atom* at1 = angle.id1.get_from(res, res2, alt))
           if (Atom* at2 = angle.id2.get_from(res, res2, alt))
             if (Atom* at3 = angle.id3.get_from(res, res2, alt)) {
-              rules.push_back({pro, RKind::Angle, angles.size()});
+              rules.push_back({RKind::Angle, angles.size()});
               angles.push_back({&angle, {{at1, at2, at3}}});
               if (!at1->altloc && !at2->altloc && !at3->altloc)
                 break;
@@ -235,7 +248,7 @@ struct Topo {
           if (Atom* at2 = tor.id2.get_from(res, res2, alt))
             if (Atom* at3 = tor.id3.get_from(res, res2, alt))
               if (Atom* at4 = tor.id4.get_from(res, res2, alt)) {
-                rules.push_back({pro, RKind::Torsion, torsions.size()});
+                rules.push_back({RKind::Torsion, torsions.size()});
                 torsions.push_back({&tor, {{at1, at2, at3, at4}}});
                 if (!at1->altloc && !at2->altloc &&
                     !at3->altloc && !at4->altloc)
@@ -247,7 +260,7 @@ struct Topo {
           if (Atom* at2 = chir.id1.get_from(res, res2, alt))
             if (Atom* at3 = chir.id2.get_from(res, res2, alt))
               if (Atom* at4 = chir.id3.get_from(res, res2, alt)) {
-                rules.push_back({pro, RKind::Chirality, chirs.size()});
+                rules.push_back({RKind::Chirality, chirs.size()});
                 chirs.push_back({&chir, {{at1, at2, at3, at4}}});
                 if (!at1->altloc && !at2->altloc &&
                     !at3->altloc && !at4->altloc)
@@ -260,7 +273,7 @@ struct Topo {
           if (Atom* atom = id.get_from(res, res2, alt))
             atoms.push_back(atom);
         if (atoms.size() >= 4) {
-          rules.push_back({pro, RKind::Plane, planes.size()});
+          rules.push_back({RKind::Plane, planes.size()});
           planes.push_back({&plane, atoms});
         }
         if (std::all_of(atoms.begin(), atoms.end(),
@@ -270,48 +283,29 @@ struct Topo {
     return rules;
   }
 
-  void apply_internal_restraints_to_residue(ResInfo& ri) {
-    auto rules = apply_restraints(ri.chemcomp.rt, *ri.res, nullptr);
-    for (const auto& rule : rules)
-      ri.rules.push_back({Provenance::Monomer, rule.rkind, rule.index});
-  }
-
   void apply_restraints_to_residue(ResInfo& ri, const MonLib& monlib) {
-    for (ResInfo::Prev& prev : ri.prev) {
-      if (const ChemLink* link = monlib.find_link(prev.link)) {
-        ResInfo* prev_ri = prev.get(&ri);
-        auto rules = apply_restraints(link->rt, *prev_ri->res, ri.res);
-        for (const auto& rule : rules) {
-          ri.rules.push_back({Provenance::PrevLink, rule.rkind, rule.index});
-          prev_ri->rules.push_back({Provenance::NextLink, rule.rkind, rule.index});
-        }
+    // link restraints
+    for (Link& prev : ri.prev)
+      if (const ChemLink* link = monlib.find_link(prev.link_id)) {
+        auto rules = apply_restraints(link->rt, *prev.res1, ri.res);
+        vector_move_extend(prev.link_rules, std::move(rules));
       }
-    }
-    apply_internal_restraints_to_residue(ri);
+    // monomer restraints
+    auto rules = apply_restraints(ri.chemcomp.rt, *ri.res, nullptr);
+    vector_move_extend(ri.monomer_rules, std::move(rules));
   }
 
-  void apply_restraints_to_extra_link(ExtraLink& link, const MonLib& monlib) {
+  void apply_restraints_to_extra_link(Link& link, const MonLib& monlib) {
     const ChemLink* cl = monlib.find_link(link.link_id);
     if (!cl) {
-      printf("Warning: ignoring link '%s' as it is not in the monomer library",
-             link.link_id.c_str());
+      err("ignoring link '" + link.link_id + "' as it is not in the monomer library");
       return;
     }
     if (link.alt1 && link.alt2 && link.alt1 != link.alt2)
-      printf("Warning: LINK between different conformers %c and %c.",
-             link.alt1, link.alt2);
+      err(tostr("LINK between different conformers ", link.alt1, " and ", link.alt2, '.'));
     char alt = link.alt1 ? link.alt1 : link.alt2;
-    ResInfo* ri1 = find_resinfo(link.res1);
-    ResInfo* ri2 = find_resinfo(link.res2);
     auto rules = apply_restraints(cl->rt, *link.res1, link.res2, alt);
-    for (Rule& rule : rules) {
-      rule.provenance = Provenance::ExtraLink;
-      link.rules.push_back(rule);
-      if (ri1)
-        ri1->rules.push_back(rule);
-      if (ri2)
-        ri2->rules.push_back(rule);
-    }
+    vector_move_extend(link.link_rules, std::move(rules));
   }
 
   // Model is non-const b/c we store non-const pointers to residues in Topo.
@@ -327,14 +321,31 @@ struct Topo {
     for (ChainInfo& chain_info : chain_infos)
       for (ResInfo& ri : chain_info.res_infos)
         apply_restraints_to_residue(ri, monlib);
-    for (ExtraLink& link : extras)
+    for (Link& link : extras)
       apply_restraints_to_extra_link(link, monlib);
+
+    // create indices
+    for (Bond& bond : bonds) {
+      bond_index.emplace(bond.atoms[0], &bond);
+      if (bond.atoms[1] != bond.atoms[0])
+        bond_index.emplace(bond.atoms[1], &bond);
+    }
+    for (Angle& ang : angles)
+      angle_index.emplace(ang.atoms[1], &ang);
+  }
+
+  GEMMI_NOINLINE void err(const std::string& msg) const {
+    if (warnings == nullptr)
+      fail(msg);
+    *warnings << "Warning: " << msg << std::endl;
   }
 };
 
-inline void Topo::ChainInfo::initialize(ResidueSpan& subchain, const Entity* ent) {
+inline Topo::ChainInfo::ChainInfo(ResidueSpan& subchain,
+                                  const Chain& chain, const Entity* ent)
+  : chain_ref(chain) {
+  subchain_name = subchain.at(0).subchain;
   res_infos.reserve(subchain.size());
-  name = subchain.at(0).subchain;
   if (ent) {
     entity_id = ent->name;
     polymer = ent->entity_type == EntityType::Polymer;
@@ -356,22 +367,24 @@ inline void Topo::ChainInfo::setup_polymer_links() {
     RGroup group = group_from(prev_group.end);
     for (auto ri = group.begin; ri != group.end; ++ri)
       for (auto prev_ri = prev_group.begin; prev_ri != prev_group.end; ++prev_ri) {
-        ResInfo::Prev p{{}, 0};
+        Link p;
+        p.res1 = prev_ri->res;
+        p.res2 = ri->res;
+        assert(prev_ri - ri == p.res1 - p.res2);
         if (are_connected(*prev_ri->res, *ri->res, polymer_type)) {
-          p.idx = int(prev_ri - ri);
           if (is_polypeptide(polymer_type)) {
             if (ri->chemcomp.group == "P-peptide")
-              p.link = "P";  // PCIS, PTRANS
+              p.link_id = "P";  // PCIS, PTRANS
             else if (ri->chemcomp.group == "M-peptide")
-              p.link = "NM"; // NMCIS, NMTRANS
-            p.link += prev_ri->res->is_cis ? "CIS" : "TRANS";
+              p.link_id = "NM"; // NMCIS, NMTRANS
+            p.link_id += prev_ri->res->is_cis ? "CIS" : "TRANS";
           } else if (is_polynucleotide(polymer_type)) {
-            p.link = "p";
+            p.link_id = "p";
           } else {
-            p.link = "?";
+            p.link_id = "?";
           }
         } else {
-          p.link = "gap";
+          p.link_id = "gap";
         }
         ri->prev.push_back(p);
       }
@@ -379,27 +392,34 @@ inline void Topo::ChainInfo::setup_polymer_links() {
   }
 }
 
-inline void Topo::ChainInfo::add_refmac_builtin_modifications() {
-  if (polymer && !res_infos.empty()) {
-    // we try to get exactly the same numbers that makecif produces
-    for (Topo::ResInfo& ri : res_infos)
-      if (polymer_type == PolymerType::PeptideL)
-        ri.mods.emplace_back("AA-STAND");
-    Topo::ResInfo& front = res_infos.front();
-    Topo::ResInfo& back = res_infos.back();
-    if (is_polypeptide(polymer_type)) {
-      if (front.chemcomp.group == "P-peptide")
-        front.mods.emplace_back("NH2");
-      else
-        front.mods.emplace_back("NH3");
-      back.mods.emplace_back(back.res->find_atom("OXT", '*') ? "COO" : "TERMINUS");
-    } else if (is_polynucleotide(polymer_type)) {
-      front.mods.emplace_back(front.res->find_atom("P", '*') ? "p5*END" : "5*END");
-      back.mods.emplace_back("TERMINUS");
-    }
-  }
+inline void Topo::ChainInfo::disable_polymer_link(const Connection& conn) {
+  const AtomAddress& a1 = conn.partner1;
+  const AtomAddress& a2 = conn.partner2;
+  if (a1.chain_name == chain_ref.name && a2.chain_name == chain_ref.name)
+    for (ResInfo& ri : res_infos)
+      for (Link& link : ri.prev) {
+        assert(link.res1 && link.res2);
+        if ((a1.res_id.matches_noseg(*link.res1) && a2.res_id.matches_noseg(*link.res2)) ||
+            (a2.res_id.matches_noseg(*link.res1) && a1.res_id.matches_noseg(*link.res2))) {
+          // don't assign the actual name here to avoid applying it twice
+          link.link_id = "?";
+          break;
+        }
+      }
 }
 
+inline Restraints::Bond bond_restraint_from_connection(const Connection& conn) {
+  Restraints::Bond bond;
+  bond.id1 = Restraints::AtomId{1, conn.partner1.atom_name};
+  bond.id2 = Restraints::AtomId{2, conn.partner2.atom_name};
+  bond.type = BondType::Unspec;
+  bond.aromatic = false;
+  bond.value = conn.reported_distance;
+  bond.esd = 0.02;
+  bond.value_nucleus = conn.reported_distance;
+  bond.esd_nucleus = 0.02;
+  return bond;
+}
 
 // Model is non-const b/c we store non-const pointers to residues in Topo.
 inline void Topo::initialize_refmac_topology(const Structure& st, Model& model0,
@@ -408,8 +428,7 @@ inline void Topo::initialize_refmac_topology(const Structure& st, Model& model0,
   for (Chain& chain : model0.chains)
     for (ResidueSpan& sub : chain.subchains()) {
       const Entity* ent = st.get_entity_of(sub);
-      chain_infos.emplace_back();
-      chain_infos.back().initialize(sub, ent);
+      chain_infos.emplace_back(sub, chain, ent);
     }
   for (ChainInfo& ci : chain_infos) {
     // copy monomer description
@@ -417,26 +436,33 @@ inline void Topo::initialize_refmac_topology(const Structure& st, Model& model0,
       auto it = monlib.monomers.find(ri.res->name);
       if (it != monlib.monomers.end())
         ri.chemcomp = it->second;
+      else
+        err("unknown chemical component " + ri.res->name
+            + " in chain " +  ci.chain_ref.name);
     }
 
     ci.setup_polymer_links();
 
-    ci.add_refmac_builtin_modifications();
+    // If a polymer links is given explicitly (LINK/struct_conn),
+    // don't use the standard link.
+    for (const Connection& conn : st.connections)
+      ci.disable_polymer_link(conn);
 
     // add modifications from standard links
     for (ResInfo& ri : ci.res_infos)
-      for (ResInfo::Prev& prev : ri.prev)
-        if (const ChemLink* link = monlib.find_link(prev.link)) {
-          prev.get(&ri)->add_mod(link->side1.mod);
+      for (Link& prev : ri.prev)
+        if (const ChemLink* link = monlib.find_link(prev.link_id)) {
+          ResInfo* ri_prev = &ri + (prev.res1 - prev.res2);
+          ri_prev->add_mod(link->side1.mod);
           ri.add_mod(link->side2.mod);
         }
   }
   // add extra links
   for (const Connection& conn : st.connections) {
-    // ignoring hydrogen bonds and metal coordination
-    if (conn.type == Connection::Hydrog || conn.type == Connection::MetalC)
+    // ignoring hydrogen bonds
+    if (conn.type == Connection::Hydrog)
       continue;
-    ExtraLink extra;
+    Link extra;
     extra.res1 = model0.find_cra(conn.partner1, true).residue;
     extra.res2 = model0.find_cra(conn.partner2, true).residue;
     if (!extra.res1 || !extra.res2)
@@ -450,8 +476,8 @@ inline void Topo::initialize_refmac_topology(const Structure& st, Model& model0,
           match->rt.bonds.empty() ||
           match->rt.bonds[0].id1.atom != conn.partner1.atom_name ||
           match->rt.bonds[0].id2.atom != conn.partner2.atom_name ||
-          !monlib.link_side_matches_residue(match->side1, *extra.res1) ||
-          !monlib.link_side_matches_residue(match->side2, *extra.res2)))
+          !monlib.link_side_matches_residue(match->side1, extra.res1->name) ||
+          !monlib.link_side_matches_residue(match->side2, extra.res2->name)))
       match = nullptr;
     // if ChemLink was not found, use the first matching link (if any)
     if (!match)
@@ -480,14 +506,8 @@ inline void Topo::initialize_refmac_topology(const Structure& st, Model& model0,
       cl.side1.comp = extra.res1->name;
       cl.side2.comp = extra.res2->name;
       cl.id = cl.side1.comp + "-" + cl.side2.comp;
-      Restraints::Bond bond;
-      bond.id1 = Restraints::AtomId{1, conn.partner1.atom_name};
-      bond.id2 = Restraints::AtomId{2, conn.partner2.atom_name};
-      bond.type = BondType::Unspec;
-      bond.aromatic = false;
-      bond.value = conn.reported_distance;
-      bond.esd = 0.02;
-      cl.rt.bonds.push_back(bond);
+      cl.rt.bonds.push_back(bond_restraint_from_connection(conn));
+
       monlib.ensure_unique_link_name(cl.id);
       monlib.links.emplace(cl.id, cl);
       extra.link_id = cl.id;
@@ -503,11 +523,11 @@ inline void Topo::initialize_refmac_topology(const Structure& st, Model& model0,
           try {
             chem_mod->apply_to(ri.chemcomp);
           } catch(std::runtime_error& e) {
-            printf("Failed to apply modification %s to %s: %s\n",
-                   chem_mod->id.c_str(), ri.res->name.c_str(), e.what());
+            err("failed to apply modification " + chem_mod->id
+                + " to " + ri.res->name + ": " + e.what());
           }
         else
-          printf("Modification not found: %s\n", modif.c_str());
+          err("modification not found: " + modif);
       }
     }
 }
