@@ -42,13 +42,18 @@ py::class_<GridBase<T>, GridMeta> add_grid_base(py::module& m, const char* name)
     });
 
   grid_base
-    .def_buffer([](GrBase &g) {
+    .def_buffer([](GrBase& g) {
       return py::buffer_info(g.data.data(),
                              {g.nu, g.nv, g.nw},       // dimensions
                              {sizeof(T),               // strides
                               sizeof(T) * g.nu,
                               sizeof(T) * g.nu * g.nv});
     })
+    .def_property_readonly("array", [](const GrBase& g) {
+      return py::array_t<T>({g.nu, g.nv, g.nw},
+                            {sizeof(T), sizeof(T) * g.nu, sizeof(T) * g.nu * g.nv},
+                            g.data.data(), py::cast(g));
+    }, py::return_value_policy::reference_internal)
     .def("point_to_index", &GrBase::point_to_index)
     .def("index_to_point", &GrBase::index_to_point)
     .def("fill", &GrBase::fill, py::arg("value"))
@@ -60,13 +65,14 @@ py::class_<GridBase<T>, GridMeta> add_grid_base(py::module& m, const char* name)
 }
 
 template<typename T>
-void add_grid(py::module& m, const std::string& name) {
+py::class_<Grid<T>, GridBase<T>> add_grid(py::module& m, const std::string& name) {
   using Gr = Grid<T>;
   using GrPoint = typename GridBase<T>::Point;
   using Masked = MaskedGrid<T>;
+  py::class_<Gr, GridBase<T>> grid(m, name.c_str());
   py::class_<Masked> masked_grid (m, ("Masked" + name).c_str());
 
-  py::class_<Gr, GridBase<T>>(m, name.c_str())
+  grid
     .def(py::init<>())
     .def(py::init([](int nx, int ny, int nz) {
       Gr* grid = new Gr();
@@ -87,15 +93,13 @@ void add_grid(py::module& m, const std::string& name) {
         grid->spacegroup = sg;
       return grid;
     }), py::arg().noconvert(), py::arg("cell")=nullptr, py::arg("spacegroup")=nullptr)
-    .def_property_readonly("spacing", [](const Gr& self){
+    .def_property_readonly("spacing", [](const Gr& self) {
         return py::make_tuple(self.spacing[0], self.spacing[1], self.spacing[2]);
     })
     .def("set_size", &Gr::set_size)
     .def("get_value", &Gr::get_value)
     .def("set_value", &Gr::set_value)
     .def("get_point", &Gr::get_point)
-    .def("get_position", &Gr::get_position)
-    .def("get_fractional", &Gr::get_fractional)
     .def("get_nearest_point", (GrPoint (Gr::*)(const Position&)) &Gr::get_nearest_point)
     .def("point_to_fractional", &Gr::point_to_fractional)
     .def("point_to_position", &Gr::point_to_position)
@@ -131,22 +135,22 @@ void add_grid(py::module& m, const std::string& name) {
     .def("symmetrize_max", &Gr::symmetrize_max)
     .def("symmetrize_abs_max", &Gr::symmetrize_abs_max)
     .def("symmetrize_sum", &Gr::symmetrize_sum)
-    .def("asu", &Gr::asu)
+    .def("masked_asu", &Gr::masked_asu, py::keep_alive<0, 1>())
     .def("mask_points_in_constant_radius", &mask_points_in_constant_radius<T>,
          py::arg("model"), py::arg("radius"), py::arg("value"))
     .def("get_subarray",
-         [](const Gr& self, int u0, int v0, int w0, int new_nu, int new_nv, int new_nw) {
-        py::array_t<T> arr({new_nu, new_nv, new_nw},
-                           {sizeof(T), sizeof(T)*new_nu, sizeof(T)*new_nu*new_nv});
-        py::buffer_info buf = arr.request();
-        T* ptr = (T*) buf.ptr;
-        int idx = 0;
-        for (int w = 0; w < new_nw; w++)
-          for (int v = 0; v < new_nv; v++)
-            for (int u = 0; u < new_nu; u++)
-              ptr[idx++] = self.get_value(u0 + u, v0 + v, w0 + w);
+         [](const Gr& self, std::array<int,3> start, std::array<int,3> shape) {
+        py::array_t<T> arr({shape[0], shape[1], shape[2]},
+                           {sizeof(T), sizeof(T)*shape[0], sizeof(T)*shape[0]*shape[1]});
+        self.get_subarray((T*) arr.request().ptr, start, shape);
         return arr;
-    })
+    }, py::arg("start"), py::arg("shape"))
+    .def("set_subarray",
+         [](Gr& self, py::array_t<T, py::array::f_style | py::array::forcecast> arr,
+            std::array<int,3> start) {
+        self.set_subarray((T*) arr.request().ptr, start,
+                          {(int)arr.shape(0), (int)arr.shape(1), (int)arr.shape(2)});
+    }, py::arg("arr"), py::arg("start"))
     .def("clone", [](const Gr& self) { return new Gr(self); })
     .def("__repr__", [=](const Gr& self) {
         return tostr("<gemmi.", name, '(', self.nu, ", ", self.nv, ", ", self.nw, ")>");
@@ -154,10 +158,16 @@ void add_grid(py::module& m, const std::string& name) {
 
   masked_grid
     .def_readonly("grid", &Masked::grid, py::return_value_policy::reference)
-    .def_readonly("mask", &Masked::mask)
+    .def_property_readonly("mask_array", [](const Masked& self) {
+      const Gr& gr = *self.grid;
+      py::array::ShapeContainer shape({gr.nu, gr.nv, gr.nw});
+      py::array::StridesContainer strides({gr.nv * gr.nw, gr.nw, 1});
+      return py::array_t<std::int8_t>(shape, strides, self.mask.data(), py::cast(self));
+    }, py::return_value_policy::reference_internal)
     .def("__iter__", [](Masked& self) { return py::make_iterator(self); },
          py::keep_alive<0, 1>())
     ;
+    return grid;
 }
 
 // for pandda-2
@@ -178,6 +188,7 @@ Grid<float> interpolate_positions(const Grid<float>& moving_map,
 
 void add_grid(py::module& m) {
   py::enum_<AxisOrder>(m, "AxisOrder")
+    .value("Unknown", AxisOrder::Unknown)
     .value("XYZ", AxisOrder::XYZ)
     .value("ZYX", AxisOrder::ZYX);
 
@@ -189,7 +200,9 @@ void add_grid(py::module& m) {
     .def_readonly("nw", &GridMeta::nw, "size in the third (slowest-changing) dim")
     .def_readonly("axis_order", &GridMeta::axis_order)
     .def_property_readonly("point_count", &GridMeta::point_count)
-    .def_property_readonly("shape", [](const GridMeta& self){
+    .def("get_position", &GridMeta::get_position)
+    .def("get_fractional", &GridMeta::get_fractional)
+    .def_property_readonly("shape", [](const GridMeta& self) {
       return py::make_tuple(self.nu, self.nv, self.nw);
     });
 
@@ -198,7 +211,9 @@ void add_grid(py::module& m) {
   add_grid_base<float>(m, "FloatGridBase")
     .def("calculate_correlation", &calculate_correlation<float>)
     ;
-  add_grid<float>(m, "FloatGrid");
+  add_grid<float>(m, "FloatGrid")
+    .def("normalize", &normalize_grid<float>)
+    ;
   add_grid_base<std::complex<float>>(m, "ComplexGridBase");
   m.def("interpolate_positions", &interpolate_positions);
 
@@ -222,6 +237,10 @@ void add_grid(py::module& m) {
     .def("put_mask_on_float_grid", &SolventMasker::put_mask_on_grid<float>)
     .def("set_to_zero", &SolventMasker::set_to_zero)
     ;
+  m.def("interpolate_grid_of_aligned_model2", &interpolate_grid_of_aligned_model2<float>,
+        py::arg("dest"), py::arg("src"), py::arg("tr"),
+        py::arg("dest_model"), py::arg("radius"));
+
 
   // from blob.hpp
   py::class_<Blob>(m, "Blob")
@@ -232,14 +251,14 @@ void add_grid(py::module& m) {
     .def_readonly("peak_pos", &Blob::peak_pos)
     ;
   m.def("find_blobs_by_flood_fill",
-        [](const Grid<float>& grid, double cutoff,
-           double min_volume, double min_score, double min_peak) {
+        [](const Grid<float>& grid, double cutoff, double min_volume,
+           double min_score, double min_peak, bool negate) {
        BlobCriteria crit;
        crit.cutoff = cutoff;
        crit.min_volume = min_volume;
        crit.min_score = min_score;
        crit.min_peak = min_peak;
-       return find_blobs_by_flood_fill(grid, crit);
-    }, py::arg("grid"), py::arg("cutoff")=true,
-       py::arg("min_volume")=10., py::arg("min_score")=15, py::arg("min_peak")=0.);
+       return find_blobs_by_flood_fill(grid, crit, negate);
+    }, py::arg("grid"), py::arg("cutoff"), py::arg("min_volume")=10.,
+       py::arg("min_score")=15., py::arg("min_peak")=0., py::arg("negate")=false);
 }
