@@ -7,7 +7,8 @@
 #include "gemmi/fprime.hpp"
 #include "gemmi/reciproc.hpp" // for count_reflections, make_miller_vector
 #include "gemmi/cif2mtz.hpp"  // for CifToMtz
-#include "gemmi/merge.hpp"  // for Intensities
+#include "gemmi/merge.hpp"    // for Intensities
+#include "gemmi/binner.hpp"   // for Binner
 
 #include "common.h"
 #include "arrvec.h"  // py_array_from_vector
@@ -162,16 +163,20 @@ void add_hkl(py::module& m) {
     })
     ;
 
-  py::class_<Intensities> intensities(m, "Intensities");
-
-  py::enum_<Intensities::Type>(intensities, "Type")
-    .value("Unknown", Intensities::Type::Unknown)
-    .value("Unmerged", Intensities::Type::Unmerged)
-    .value("Mean", Intensities::Type::Mean)
-    .value("Anomalous", Intensities::Type::Anomalous)
+  py::enum_<DataType>(m, "DataType")
+    .value("Unknown", DataType::Unknown)
+    .value("Unmerged", DataType::Unmerged)
+    .value("Mean", DataType::Mean)
+    .value("Anomalous", DataType::Anomalous)
     ;
+  m.def("check_data_type_under_symmetry", [](const ReflnBlock& data) {
+      return check_data_type_under_symmetry(ReflnDataProxy(data));
+  });
+  m.def("check_data_type_under_symmetry", [](const Mtz& data) {
+      return check_data_type_under_symmetry(MtzDataProxy{data});
+  });
 
-  intensities
+  py::class_<Intensities>(m, "Intensities")
     .def(py::init<>())
     .def_readwrite("spacegroup", &Intensities::spacegroup)
     .def_readwrite("unit_cell", &Intensities::unit_cell)
@@ -211,16 +216,16 @@ void add_hkl(py::module& m) {
       return py::array_t<short>({(py::ssize_t)self.data.size()}, {stride},
                                 &data->isign, py::cast(self));
     }, py::return_value_policy::reference_internal)
-    .def("set_data", [](Intensities& self, const UnitCell& unit_cell, 
-                        const SpaceGroup* sg, py::array_t<int> hkl, 
+    .def("set_data", [](Intensities& self, const UnitCell& unit_cell,
+                        const SpaceGroup* sg, py::array_t<int> hkl,
                         py::array_t<double> values, py::array_t<double> sigmas) {
       auto h = hkl.unchecked<2>();
       if (h.shape(1) != 3)
-        throw std::domain_error("error: the size of the second dimension != 3");
+        throw std::domain_error("the hkl array must have size N x 3");
       auto v = values.template unchecked<1>();
       auto s = sigmas.template unchecked<1>();
       if (h.shape(0) != v.shape(0) || h.shape(0) != s.shape(0))
-        throw std::domain_error("error: arrays have different lengths");
+        throw std::domain_error("arrays have different lengths");
 
       self.unit_cell = unit_cell;
       self.spacegroup = sg;
@@ -230,8 +235,82 @@ void add_hkl(py::module& m) {
           self.data.push_back({{{h(i, 0), h(i, 1), h(i, 2)}}, 1, 0, v(i), s(i)});
 
       self.switch_to_asu_indices();
-      self.type = Intensities::Type::Unmerged;
+      self.type = DataType::Unmerged;
     }, py::arg("cell"), py::arg("sg").none(false),
        py::arg("miller_array"), py::arg("value_array"), py::arg("sigma_array"))
     ;
+
+  py::class_<Binner> binner(m, "Binner");
+  py::enum_<Binner::Method>(binner, "Method")
+      .value("EqualCount", Binner::Method::EqualCount)
+      .value("Dstar", Binner::Method::Dstar)
+      .value("Dstar2", Binner::Method::Dstar2)
+      .value("Dstar3", Binner::Method::Dstar3)
+      ;
+  binner
+    .def(py::init<>())
+    .def("setup", [](Binner& self, int nbins, Binner::Method method,
+                     const Mtz& mtz, const UnitCell* cell) {
+        return self.setup(nbins, method, MtzDataProxy{mtz}, cell);
+    }, py::arg("nbins"), py::arg("method"), py::arg("mtz"), py::arg("cell")=nullptr)
+    .def("setup", [](Binner& self, int nbins, Binner::Method method,
+                     const ReflnBlock& r, const UnitCell* cell) {
+        return self.setup(nbins, method, ReflnDataProxy(r), cell);
+    }, py::arg("nbins"), py::arg("method"), py::arg("r"), py::arg("cell")=nullptr)
+    .def("setup", [](Binner& self, int nbins, Binner::Method method,
+                     py::array_t<int> hkl, const UnitCell* cell) {
+        auto h = hkl.unchecked<2>();
+        if (h.shape(1) != 3)
+          throw std::domain_error("the hkl array must have size N x 3");
+        std::vector<double> inv_d2(h.shape(0));
+        if (cell)
+          for (size_t i = 0; i < inv_d2.size(); ++i)
+            inv_d2[i] = cell->calculate_1_d2_double(h(i, 0), h(i, 1), h(i, 2));
+        return self.setup_from_1_d2(nbins, method, std::move(inv_d2), cell);
+    }, py::arg("nbins"), py::arg("method"), py::arg("hkl"), py::arg("cell"))
+    .def("setup_from_1_d2", [](Binner& self, int nbins, Binner::Method method,
+                               py::array_t<double> inv_d2, const UnitCell* cell) {
+        double* ptr = (double*) inv_d2.request().ptr;
+        auto len = inv_d2.shape(0);
+        return self.setup_from_1_d2(nbins, method, std::vector<double>(ptr, ptr+len), cell);
+    }, py::arg("nbins"), py::arg("method"), py::arg("inv_d2"), py::arg("cell"))
+    .def("get_bin", &Binner::get_bin)
+    .def("get_bins", [](Binner& self, const Mtz& mtz) {
+        return py_array_from_vector(self.get_bins(MtzDataProxy{mtz}));
+    })
+    .def("get_bins", [](Binner& self, const ReflnBlock& r) {
+        return py_array_from_vector(self.get_bins(ReflnDataProxy(r)));
+    })
+    .def("get_bins", [](Binner& self, py::array_t<int> hkl) {
+        self.ensure_limits_are_set();
+        auto h = hkl.unchecked<2>();
+        if (h.shape(1) != 3)
+          throw std::domain_error("the hkl array must have size N x 3");
+        auto len = h.shape(0);
+        int hint = 0;
+        py::array_t<int> arr(len);
+        int* ptr = (int*) arr.request().ptr;
+        for (int i = 0; i < len; ++i)
+          ptr[i] = self.get_bin_hinted({{h(i, 0), h(i, 1), h(i, 2)}}, hint);
+        return arr;
+    })
+    .def("get_bins_from_1_d2", [](Binner& self, py::array_t<double> inv_d2) {
+        self.ensure_limits_are_set();
+        auto v = inv_d2.unchecked<1>();
+        auto len = v.shape(0);
+        int hint = 0;
+        py::array_t<int> arr(len);
+        int* ptr = (int*) arr.request().ptr;
+        for (int i = 0; i < len; ++i)
+          ptr[i] = self.get_bin_from_1_d2_hinted(v(i), hint);
+        return arr;
+    })
+    .def("dmin_of_bin", &Binner::dmin_of_bin)
+    .def("dmax_of_bin", &Binner::dmax_of_bin)
+    .def_property_readonly("size", &Binner::size)
+    .def_readonly("limits", &Binner::limits)
+    .def_readwrite("cell", &Binner::cell)
+    .def_readonly("min_1_d2", &Binner::min_1_d2)
+    .def_readonly("max_1_d2", &Binner::max_1_d2)
+    ;
 }
