@@ -2,6 +2,7 @@
 
 #include "gemmi/model.hpp"
 #include "gemmi/calculate.hpp"  // for calculate_mass, count_atom_sites
+#include "gemmi/modify.hpp"     // for remove_alternative_conformations
 #include "gemmi/polyheur.hpp"   // for one_letter_code, trim_to_alanine
 #include "gemmi/assembly.hpp"   // for expand_ncs, HowToNameCopiedChain
 #include "gemmi/tostr.hpp"
@@ -68,7 +69,7 @@ void add_mol(py::module& m) {
 
   py::enum_<CoorFormat>(m, "CoorFormat")
     .value("Unknown", CoorFormat::Unknown)
-    .value("UnknownAny", CoorFormat::UnknownAny)
+    .value("Detect", CoorFormat::Detect)
     .value("Pdb", CoorFormat::Pdb)
     .value("Mmcif", CoorFormat::Mmcif)
     .value("Mmjson", CoorFormat::Mmjson)
@@ -84,6 +85,7 @@ void add_mol(py::module& m) {
     .def_readwrite("spacegroup_hm", &Structure::spacegroup_hm)
     .def_readwrite("ncs", &Structure::ncs)
     .def_readwrite("resolution", &Structure::resolution)
+    .def_readwrite("input_format", &Structure::input_format)
     .def_readwrite("entities", &Structure::entities)
     .def_readwrite("connections", &Structure::connections)
     .def_readwrite("helices", &Structure::helices)
@@ -134,8 +136,7 @@ void add_mol(py::module& m) {
          remove_alternative_conformations<Structure>)
     .def("remove_hydrogens", remove_hydrogens<Structure>)
     .def("remove_waters", remove_waters<Structure>)
-    .def("remove_ligands_and_waters",
-         (void (*)(Structure&)) &remove_ligands_and_waters)
+    .def("remove_ligands_and_waters", remove_ligands_and_waters<Structure>)
     .def("shorten_chain_names", &shorten_chain_names)
     .def("expand_ncs", &expand_ncs, py::arg("how"))
     .def("calculate_box", &calculate_box, py::arg("margin")=0.)
@@ -213,6 +214,10 @@ void add_mol(py::module& m) {
     .def("__delitem__", &Model::remove_chain, py::arg("name"))
     .def("__delitem__", remove_child<Model>, py::arg("index"))
     .def("__delitem__", remove_children<Model>)
+    .def("remove_alternative_conformations", remove_alternative_conformations<Model>)
+    .def("remove_hydrogens", remove_hydrogens<Model>)
+    .def("remove_waters", remove_waters<Model>)
+    .def("remove_ligands_and_waters", remove_ligands_and_waters<Model>)
     .def("count_atom_sites", &count_atom_sites<Model>)
     .def("count_hydrogen_sites", &count_hydrogen_sites<Model>)
     .def("count_occupancies", &count_occupancies<Model>)
@@ -220,10 +225,9 @@ void add_mol(py::module& m) {
     .def("calculate_center_of_mass", [](const Model& self) {
         return calculate_center_of_mass(self).get();
     })
-    .def("transform", [](Model& self, const Transform& tr) {
-        for (CRA cra : self.all())
-          transform_atom(*cra.atom, tr);
-    }, py::arg("tr"))
+    .def("transform_pos_and_adp", transform_pos_and_adp<Model>, py::arg("tr"))
+    // the same function under differnt name - for compability
+    .def("transform", transform_pos_and_adp<Model>, py::arg("tr"))
     .def("split_chains_by_segments", &split_chains_by_segments)
     .def("clone", [](const Model& self) { return new Model(self); })
     .def("__repr__", [](const Model& self) {
@@ -337,9 +341,17 @@ void add_mol(py::module& m) {
     .def("make_one_letter_sequence", [](const ResidueSpan& span) {
         return make_one_letter_sequence(span);
     })
+    .def("transform_pos_and_adp", transform_pos_and_adp<ResidueSpan>)
     .def("__repr__", [](const ResidueSpan& self) {
         int N = (int) self.size();
-        std::string r = "<gemmi.ResidueSpan of " + std::to_string(N) + ": [";
+        std::string r = "<gemmi.ResidueSpan of " + std::to_string(N) + ": ";
+        if (N > 0) {
+          r += self.front().subchain;
+          if (self.back().subchain != self.front().subchain)
+            r += " - " + self.back().subchain;
+          r += ' ';
+        }
+        r += '[';
         for (int i = 0; i < (N < 5 ? N : 3); ++i) {
           if (i != 0) r += ' ';
           r += self[i].str();
@@ -382,6 +394,7 @@ void add_mol(py::module& m) {
   pyResidue
     .def(py::init<>())
     .def_readwrite("subchain", &Residue::subchain)
+    .def_readonly("entity_id", &Residue::entity_id)
     .def_readwrite("entity_type", &Residue::entity_type)
     .def_readwrite("het_flag", &Residue::het_flag)
     .def_readwrite("flag", &Residue::flag)
