@@ -125,19 +125,19 @@ void print_contacts(Structure& st, const ContactParameters& params) {
       std::string conn_info;
       if (Connection* conn = st.find_connection_by_cra(cra1, cra2))
         conn_info = conn->name.empty() ? "(link)" : conn->name;
-      printf("%-11s %-4s%c%3s%2s%5s         "
-             "      %-4s%c%3s%2s%5s  %6s %6s %5.2f\n",
+      printf("%-11s %-4s%c%3s%2s%4s%c         "
+             "      %-4s%c%3s%2s%4s%c  %6s %6s %5.2f\n",
              conn_info.c_str(),
              cra1.atom->padded_name().c_str(),
              cra1.atom->altloc ? std::toupper(cra1.atom->altloc) : ' ',
              cra1.residue->name.c_str(),
              cra1.chain->name.c_str(),
-             cra1.residue->seqid.str().c_str(),
+             cra1.residue->seqid.num.str().c_str(), cra1.residue->seqid.icode,
              cra2.atom->padded_name().c_str(),
              cra2.atom->altloc ? std::toupper(cra2.atom->altloc) : ' ',
              cra2.residue->name.c_str(),
              cra2.chain->name.c_str(),
-             cra2.residue->seqid.str().c_str(),
+             cra2.residue->seqid.num.str().c_str(), cra2.residue->seqid.icode,
              sym1.c_str(), sym2.c_str(), std::sqrt(dist_sq));
   });
   if (params.print_count)
