@@ -3,8 +3,6 @@
 #include <cstdio>   // for printf, fprintf
 #include <cstdlib>  // for atof
 #include "gemmi/ccp4.hpp"
-#include "gemmi/floodfill.hpp" // for remove_islands_in_mask
-#include "gemmi/gz.hpp"        // for MaybeGzipped
 #include "gemmi/solmask.hpp"   // for SolventMasker
 #include "gemmi/symmetry.hpp"
 #include "gemmi/read_coor.hpp" // for read_structure_gz
