@@ -23,7 +23,7 @@ class TestMath(unittest.TestCase):
         self.assertEqual(a.tolist(), [-3, 3, 13])
 
     def test_SMat33_transformed_by(self):
-        tensor = gemmi.SMat33f(random(), random(), random(),
+        tensor = gemmi.SMat33d(random(), random(), random(),
                                random(), random(), random())
         mat = gemmi.Mat33()
         mat.fromlist([[random() for _ in range(3)] for _ in range(3)])
@@ -192,7 +192,6 @@ class TestGruber(unittest.TestCase):
         t = 108.5325886
         par = (p*p, p*p, q*q, -p*p, -p*p, 0)
         assert_almost_equal_seq(self, gv.parameters, par)
-        print(gv.cell_parameters())
         self.assertTrue(gv.cell_parameters(), (p, p, q, t, t, 0))
 
     def test_near_degenerate(self):
